import os
import tempfile
import re
from os.path import join, basename

from Products.PortalTransforms.interfaces import itransform
from utils import bin_search, sansext

class commandtransform:
    """abstract class for external command based transform
    """
    __implements__ = itransform

    def __init__(self, name=None, binary=None, **kwargs):
        if name is not None:
            self.__name__ = name
        if binary is not None:
            self.binary = bin_search(binary)

    def name(self):
        return self.__name__

    def initialize_tmpdir(self, data, **kwargs):
        """create a temporary directory, copy input in a file there
        return the path of the tmp dir and of the input file
        """
        tmpdir = tempfile.mktemp()
        os.mkdir(tmpdir)
        filename = kwargs.get("filename", '')
        fullname = os.path.join(tmpdir, basename(filename))
        filedest = open(fullname , "wb")
        filedest.write(data)
        filedest.close()
        return tmpdir, fullname

    def subObjects(self, tmpdir):
        imgs = []
        for f in os.listdir(tmpdir):
            result = re.match("^.+\.(?P<ext>.+)$", f)
            if result is not None:
                ext = result.group('ext')
                if ext in ('png', 'jpg', 'gif'):
                    imgs.append(f)
        path = os.path.join(tmpdir, '')
        return path, imgs

    def fixImages(self, path, images, objects):
        for image in images:
            file = open(join(path, image), 'rb')
            data = file.read()
            file.close()
            objects[image] = data

    def cleanDir(self, tmpdir):
        for f in os.listdir("%s" % tmpdir):
            os.remove(os.path.join(tmpdir, f))
        os.rmdir("%s" % tmpdir)
